@@ -7,8 +7,6 @@ import itertools, warnings
 
 from _HTSeq import *
 
-from _HTSeq_internal import peek
-
 from _version import __version__
 
 
@@ -38,6 +36,7 @@ class FileOrSequence( object ):
       self.line_no = None
       
    def __iter__( self ):
+      self.line_no = 1
       if isinstance( self.fos, str ):
          if self.fos.lower().endswith( ( ".gz" , ".gzip" ) ):
             lines = gzip.open( self.fos )
@@ -45,7 +44,6 @@ class FileOrSequence( object ):
             lines = open( self.fos )
       else:
          lines = self.fos
-      self.line_no = 1
       for line in lines:
          yield line
          self.line_no += 1
@@ -71,7 +69,7 @@ class FileOrSequence( object ):
          return "line %d of file %s" % ( self.line_no, self.fos )
       else:
          return "line %d" % self.line_no
-
+         
 class GeneratedSequence( object ):
    """GeneratedSequence is a little helper class to get "respawnable"
    iterator generators. Pass its constructor a function that generates
@@ -161,6 +159,7 @@ def parse_GFF_attribute_string( attrStr, extra_return_first_value=False ):
    if attrStr.endswith( "\n" ):
       attrStr = attrStr[:-1]
    d = {}
+   first_val = "_unnamed_"
    for (i, attr) in itertools.izip( itertools.count(), attrStr.split( ";" ) ):
       if _re_attr_empty.match( attr ):
          continue
@@ -202,6 +201,8 @@ class GFF_Reader( FileOrSequence ):
    
    def __iter__( self ):
       for line in FileOrSequence.__iter__( self ):
+         if line == "\n":
+            continue
          if line.startswith( '#' ):
             if line.startswith( "##" ):
                mo = _re_gff_meta_comment.match( line )
@@ -492,7 +493,15 @@ class SAM_Reader( FileOrSequence ):
    contains short read alignments. It can generate an iterator of Alignment
    objects."""
 
+   def __init__( self, filename_or_sequence ):      
+      FileOrSequence.__init__( self, filename_or_sequence )
+      self.peeked = None      
+
    def __iter__( self ):
+      if self.peeked is not None and self.peeked != []:
+         for read in self.peeked:
+            yield read
+         self.peeked = None
       for line in FileOrSequence.__iter__( self ):
          if line.startswith( "@" ):
             # do something with the header line
@@ -504,7 +513,15 @@ class SAM_Reader( FileOrSequence ):
             raise
          yield algnt
        
-
+   def peek( self, num = 1 ):
+      i = 0
+      self.peeked = []
+      for read in self:
+         self.peeked.append( read )
+         i += 1
+         if i >= num:
+            break
+      return self.peeked
 
 class GenomicArrayOfSets( GenomicArray ):
 
@@ -548,15 +565,10 @@ def pair_SAM_alignments( alignments ):
          for a2 in almnt_list:
             if a1.pe_which == a2.pe_which:
                continue
-            if not a1.aligned and not a2.mate_aligned:
-               break
-            if not a1.mate_aligned and not a2.aligned:
-               break
-            if a1.aligned and not a2.mate_aligned:
+            if a1.aligned != a2.mate_aligned or a1.mate_aligned != a2.aligned:
                continue
-            if a1.mate_aligned and not a2.aligned:
-               continue
-            assert a1.aligned and a2.aligned
+            if not (a1.aligned and a2.aligned):
+               break
             if a1.iv.chrom == a2.mate_start.chrom and a1.iv.start == a2.mate_start.pos and \
                   a2.iv.chrom == a1.mate_start.chrom and a2.iv.start == a1.mate_start.pos:
                break
